/* libhtmlparse: Quick and simple HTML parser
   Mooneer Salem (mooneer@translator.cx) */

#ifndef LIBHTML_H
#define LIBHTML_H

/****************
* from the guts *
*****************/

/* struct holding the arguments of tags */

struct ArgvTable {
	char *arg, *val;
};

/* struct for streamparsing and errorhandling */

struct ErrorTree {
	unsigned int line;
	unsigned char type;
	char *problem;
};

/* obligatory prototypes */

static const char *parseText     	(const char *);
static const char *parseComment  	(const char *);
static const char *parseEndTag   	(const char *);
static const char *parsePHP      	(const char *);
static const char *parseXMLtag   	(const char *);
static const char *eatUp         	(const char *);
static const char *eatUpText     	(const char *);
static const char *parseXML      	(const char *);
static const char *parseStartTag 	(const char *);
static const char *parseDecl     	(const char *);
static const char *parseForEntities	(const char *);

/**********************************************************************/

/***********************************************************************
*                        The CallBacks                                 *
* You may call one ore several or even all callbacks. Except of the    *
* XHTMLCallBack, all CallBacks will work as expected and described     *
*                                                                      *
* XHTMLCallBack:                                                       *
* The XHTMLCallBack is a special case, because you can decide, if the  *
* XHTML specific tags should be handeled as a start- AND endtag, or    *
* as an XHTML tag. If you call nothing, except start and endtag, the   *
* behaviour is, that you'll get a start AND an endtag called back.     *
* If you call XHTMLCallBack, it will only give you the XHTML call back.*
*                                                                      *
* If you are in doubt or simply confused now, call XHTMLCallBack()     *
************************************************************************/

/* previously, the global variables below were declared non-extern which,
 * on Mac OS X, seemed to mean that each module got its own copy of these
 * variables, which would in turn be illegal.  Hence we were getting
 * crazy "multiple definition" errors from the linker.  The correct way
 * of expressing all of this is to say that the variables are extern in
 * all modules but one, where that one module is where the space for the
 * variables is actually allocated.  This seems to make no difference on
 * Linux, but on Mac OS X, it makes the code compile and link.
 *
 * Note that the way that I do this is somewhat of a hack.
 * IN_LIBHTMLPARSE_C is defined in libhtmlparse.c and no where else.
 * This works, and saves on duplication, but a cleaner, more conventional
 * way of doing this would be to re-define all of these in libhtmlparse.c
 * but without the 'extern'.
 * -- Filip Pizlo 2003 */
#ifdef IN_LIBHTMLPARSE_C
#define extern
#endif

/* handle comments and javascript */
extern int (*commentCallBack) (char *comment);
extern int (*commentStartCallBack) ();
extern int (*commentEndCallBack) ();

/* Declaration e.g. <!DOCTYPE HTML ... */
extern int (*declCallBack) (char *tag, /*@null@*/ struct ArgvTable *args, int numargs);

/* Start tag e.g. <html>, with arguments, args may be NULL, numargs may be 0 */
extern int (*startCallBack) (char *tag, /*@null@*/ struct ArgvTable *args, int numargs);

/* End tag e.g. </html>*/
extern int (*endCallBack) (char *tag);

/* handle plain text */
extern int (*textCallBack) (char *text);
extern int (*textStartCallBack) ();
extern int (*textEndCallBack) ();

/* PHP inserts. BUG(?): if someone prints another PHP function from this PHP function
   our lib will get confused. */
extern int (*phpCallBack) (char *text);

/* empty tags like <hr/>, <br/>, with arguments, args may be NULL, numargs may be 0 */
extern int (*XHTMLCallBack) (char *tag, /*@null@*/ struct ArgvTable *args, int numargs);

/* XML tags <?xml>, with arguments, args may be NULL, numargs may be 0 */
extern int (*xmlCallBack) (char *tag, /*@null@*/ struct ArgvTable *args, int numargs);

/* entities like &auml;,&#228; text will inherit all chars between '&' and ';' */

extern int (*entityCallBack) (char *text);

#ifdef IN_LIBHTMLPARSE_C
#undef extern
#endif

/* the script inside <script> ... </script> */
/* int (*scriptCallBack) (char *text);  */

/* parse the inputfile */
/*@null@*/ struct ErrorTree *parse (const char *html);

/**********************************************************************/

/* more of the obligatory prototypes ( one should clean up here! ;-) */

/* puts found arguments into the argument struct */
static struct ArgvTable *addArgToTable (/*@null@*/ struct ArgvTable *args, char *arg, /*@null@*/ char *val);

/* cleans the argument struct */
static void freeArgs (/*@null@*/ struct ArgvTable *args);

#endif /* LIBHTML_H */

